@@ -1,125 +1,131 @@
-{-|
-Module     : Data.Ini.Config
-Copyright  : (c) Getty Ritter, 2017
-License    : BSD
-Maintainer : Getty Ritter <config-ini@infinitenegativeutility.com>
-Stability  : experimental
-
-The 'config-ini' library exports some simple monadic functions to
-make parsing INI-like configuration easier. INI files have a
-two-level structure: the top-level named chunks of configuration,
-and the individual key-value pairs contained within those chunks.
-For example, the following INI file has two sections, @NETWORK@
-and @LOCAL@, and each contains its own key-value pairs. Comments,
-which begin with @#@ or @;@, are ignored:
-
-> [NETWORK]
-> host = example.com
-> port = 7878
->
-> # here is a comment
-> [LOCAL]
-> user = terry
-
-The combinators provided here are designed to write quick and
-idiomatic parsers for files of this form. Sections are parsed by
-'IniParser' computations, like 'section' and its variations,
-while the fields within sections are parsed by 'SectionParser'
-computations, like 'field' and its variations. If we want to
-parse an INI file like the one above, treating the entire
-@LOCAL@ section as optional, we can write it like this:
-
-> data Config = Config
->   { cfNetwork :: NetworkConfig, cfLocal :: Maybe LocalConfig }
->     deriving (Eq, Show)
->
-> data NetworkConfig = NetworkConfig
->   { netHost :: String, netPort :: Int }
->     deriving (Eq, Show)
->
-> data LocalConfig = LocalConfig
->   { localUser :: Text }
->     deriving (Eq, Show)
->
-> configParser :: IniParser Config
-> configParser = do
->   netCf <- section "NETWORK" $ do
->     host <- fieldOf "host" string
->     port <- fieldOf "port" number
->     return NetworkConfig { netHost = host, netPort = port }
->   locCf <- sectionMb "LOCAL" $
->     LocalConfig <$> field "user"
->   return Config { cfNetwork = netCf, cfLocal = locCf }
-
-
-We can run our computation with 'parseIniFile', which,
-when run on our example file above, would produce the
-following:
-
->>> parseIniFile example configParser
-Right (Config {cfNetwork = NetworkConfig {netHost = "example.com", netPort = 7878}, cfLocal = Just (LocalConfig {localUser = "terry"})})
-
--}
-
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 
+-- |
+-- Module     : Data.Ini.Config
+-- Copyright  : (c) Getty Ritter, 2017
+-- License    : BSD
+-- Maintainer : Getty Ritter <config-ini@infinitenegativeutility.com>
+-- Stability  : experimental
+--
+-- The 'config-ini' library exports some simple monadic functions to
+-- make parsing INI-like configuration easier. INI files have a
+-- two-level structure: the top-level named chunks of configuration,
+-- and the individual key-value pairs contained within those chunks.
+-- For example, the following INI file has two sections, @NETWORK@
+-- and @LOCAL@, and each contains its own key-value pairs. Comments,
+-- which begin with @#@ or @;@, are ignored:
+--
+-- > [NETWORK]
+-- > host = example.com
+-- > port = 7878
+-- >
+-- > # here is a comment
+-- > [LOCAL]
+-- > user = terry
+--
+-- The combinators provided here are designed to write quick and
+-- idiomatic parsers for files of this form. Sections are parsed by
+-- 'IniParser' computations, like 'section' and its variations,
+-- while the fields within sections are parsed by 'SectionParser'
+-- computations, like 'field' and its variations. If we want to
+-- parse an INI file like the one above, treating the entire
+-- @LOCAL@ section as optional, we can write it like this:
+--
+-- > data Config = Config
+-- >   { cfNetwork :: NetworkConfig, cfLocal :: Maybe LocalConfig }
+-- >     deriving (Eq, Show)
+-- >
+-- > data NetworkConfig = NetworkConfig
+-- >   { netHost :: String, netPort :: Int }
+-- >     deriving (Eq, Show)
+-- >
+-- > data LocalConfig = LocalConfig
+-- >   { localUser :: Text }
+-- >     deriving (Eq, Show)
+-- >
+-- > configParser :: IniParser Config
+-- > configParser = do
+-- >   netCf <- section "NETWORK" $ do
+-- >     host <- fieldOf "host" string
+-- >     port <- fieldOf "port" number
+-- >     return NetworkConfig { netHost = host, netPort = port }
+-- >   locCf <- sectionMb "LOCAL" $
+-- >     LocalConfig <$> field "user"
+-- >   return Config { cfNetwork = netCf, cfLocal = locCf }
+--
+--
+-- We can run our computation with 'parseIniFile', which,
+-- when run on our example file above, would produce the
+-- following:
+--
+-- >>> parseIniFile example configParser
+-- Right (Config {cfNetwork = NetworkConfig {netHost = "example.com", netPort = 7878}, cfLocal = Just (LocalConfig {localUser = "terry"})})
 module Data.Ini.Config
-(
--- * Parsing Files
-  parseIniFile
--- * Parser Types
-, IniParser
-, SectionParser
--- * Section-Level Parsing
-, section
-, sections
-, sectionOf
-, sectionsOf
-, sectionMb
-, sectionDef
--- * Field-Level Parsing
-, field
-, fieldOf
-, fieldMb
-, fieldMbOf
-, fieldDef
-, fieldDefOf
-, fieldFlag
-, fieldFlagDef
--- * Reader Functions
-, readable
-, number
-, string
-, flag
-, listWithSeparator
-) where
+  ( -- * Parsing Files
+    parseIniFile,
 
-import           Control.Applicative (Alternative(..))
-import           Control.Monad.Trans.Except
-import           Data.Ini.Config.Raw
-import           Data.Sequence (Seq)
+    -- * Parser Types
+    IniParser,
+    SectionParser,
+
+    -- * Section-Level Parsing
+    section,
+    sections,
+    sectionOf,
+    sectionsOf,
+    sectionMb,
+    sectionDef,
+
+    -- * Field-Level Parsing
+    field,
+    fieldOf,
+    fieldMb,
+    fieldMbOf,
+    fieldDef,
+    fieldDefOf,
+    fieldFlag,
+    fieldFlagDef,
+
+    -- * Reader Functions
+    readable,
+    number,
+    string,
+    flag,
+    listWithSeparator,
+  )
+where
+
+import Control.Applicative (Alternative (..))
+import Control.Monad.Trans.Except
+import Data.Ini.Config.Raw
+import Data.Sequence (Seq)
 import qualified Data.Sequence as Seq
-import           Data.String (IsString(..))
-import           Data.Text (Text)
+import Data.String (IsString (..))
+import Data.Text (Text)
 import qualified Data.Text as T
-import           Data.Typeable (Typeable, Proxy(..), typeRep)
-import           GHC.Exts (IsList(..))
-import           Text.Read (readMaybe)
+import Data.Typeable (Proxy (..), Typeable, typeRep)
+import GHC.Exts (IsList (..))
+import Text.Read (readMaybe)
 
 lkp :: NormalizedText -> Seq (NormalizedText, a) -> Maybe a
 lkp t = go . Seq.viewl
-  where go ((t', x) Seq.:< rs)
-          | t == t'   = Just x
-          | otherwise = go (Seq.viewl rs)
-        go Seq.EmptyL = Nothing
+  where
+    go ((t', x) Seq.:< rs)
+      | t == t' = Just x
+      | otherwise = go (Seq.viewl rs)
+    go Seq.EmptyL = Nothing
 
 addLineInformation :: Int -> Text -> StParser s a -> StParser s a
 addLineInformation lineNo sec = withExceptT go
-  where go e = "Line " ++ show lineNo ++
-               ", in section " ++ show sec ++
-               ": " ++ e
+  where
+    go e =
+      "Line " ++ show lineNo
+        ++ ", in section "
+        ++ show sec
+        ++ ": "
+        ++ e
 
 type StParser s a = ExceptT String ((->) s) a
 
@@ -149,10 +155,11 @@ parseIniFile text (IniParser mote) = do
 --   >>> parseIniFile "[ONE]\nx = hello\n" $ section "TWO" (field "x")
 --   Left "No top-level section named \"TWO\""
 section :: Text -> SectionParser a -> IniParser a
-section name (SectionParser thunk) = IniParser $ ExceptT $ \(RawIni ini) ->
-  case lkp (normalize name) ini of
-    Nothing  -> Left ("No top-level section named " ++ show name)
-    Just sec -> runExceptT thunk sec
+section name (SectionParser thunk) = IniParser $
+  ExceptT $ \(RawIni ini) ->
+    case lkp (normalize name) ini of
+      Nothing -> Left ("No top-level section named " ++ show name)
+      Just sec -> runExceptT thunk sec
 
 -- | Find multiple named sections in the INI file and parse them all
 --   with the provided section parser. In order to support classic INI
@@ -164,10 +171,12 @@ section name (SectionParser thunk) = IniParser $ ExceptT $ \(RawIni ini) ->
 --   >>> parseIniFile "[ONE]\nx = hello\n" $ sections "TWO" (field "x")
 --   Right (fromList [])
 sections :: Text -> SectionParser a -> IniParser (Seq a)
-sections name (SectionParser thunk) = IniParser $ ExceptT $ \(RawIni ini) ->
-  let name' = normalize name
-  in mapM (runExceptT thunk . snd)
-          (Seq.filter (\ (t, _) -> t == name') ini)
+sections name (SectionParser thunk) = IniParser $
+  ExceptT $ \(RawIni ini) ->
+    let name' = normalize name
+     in mapM
+          (runExceptT thunk . snd)
+          (Seq.filter (\(t, _) -> t == name') ini)
 
 -- | A call to @sectionOf f@ will apply @f@ to each section name and,
 --   if @f@ produces a "Just" value, pass the extracted value in order
@@ -180,15 +189,15 @@ sections name (SectionParser thunk) = IniParser $ ExceptT $ \(RawIni ini) ->
 --   >>> parseIniFile "[BAR]\nx = hello\n" $ sectionOf (T.stripSuffix "OO") (\ l -> fmap ((,) l) (field "x"))
 --   Left "No matching top-level section"
 sectionOf :: (Text -> Maybe b) -> (b -> SectionParser a) -> IniParser a
-sectionOf fn sectionParser = IniParser $ ExceptT $ \(RawIni ini) ->
-  let go Seq.EmptyL = Left "No matching top-level section"
-      go ((t, sec) Seq.:< rs)
-        | Just v <- fn (actualText t) =
+sectionOf fn sectionParser = IniParser $
+  ExceptT $ \(RawIni ini) ->
+    let go Seq.EmptyL = Left "No matching top-level section"
+        go ((t, sec) Seq.:< rs)
+          | Just v <- fn (actualText t) =
             let SectionParser thunk = sectionParser v
-            in runExceptT thunk sec
-        | otherwise = go (Seq.viewl rs)
-  in go (Seq.viewl ini)
-
+             in runExceptT thunk sec
+          | otherwise = go (Seq.viewl rs)
+     in go (Seq.viewl ini)
 
 -- | A call to @sectionsOf f@ will apply @f@ to each section name and,
 --   if @f@ produces a @Just@ value, pass the extracted value in order
@@ -201,17 +210,18 @@ sectionOf fn sectionParser = IniParser $ ExceptT $ \(RawIni ini) ->
 --   >>> parseIniFile "[BAR]\nx = hello\n" $ sectionsOf (T.stripSuffix "OO") (\ l -> fmap ((,) l) (field "x"))
 --   Right (fromList [])
 sectionsOf :: (Text -> Maybe b) -> (b -> SectionParser a) -> IniParser (Seq a)
-sectionsOf fn sectionParser = IniParser $ ExceptT $ \(RawIni ini) ->
-  let go Seq.EmptyL = return Seq.empty
-      go ((t, sec) Seq.:< rs)
-        | Just v <- fn (actualText t) =
+sectionsOf fn sectionParser = IniParser $
+  ExceptT $ \(RawIni ini) ->
+    let go Seq.EmptyL = return Seq.empty
+        go ((t, sec) Seq.:< rs)
+          | Just v <- fn (actualText t) =
             let SectionParser thunk = sectionParser v
-            in do
-              x <- runExceptT thunk sec
-              xs <- go (Seq.viewl rs)
-              return (x Seq.<| xs)
-        | otherwise = go (Seq.viewl rs)
-  in go (Seq.viewl ini)
+             in do
+                  x <- runExceptT thunk sec
+                  xs <- go (Seq.viewl rs)
+                  return (x Seq.<| xs)
+          | otherwise = go (Seq.viewl rs)
+     in go (Seq.viewl ini)
 
 -- | Find a named section in the INI file and parse it with the provided
 --   section parser, returning 'Nothing' if the section does not exist.
@@ -224,10 +234,11 @@ sectionsOf fn sectionParser = IniParser $ ExceptT $ \(RawIni ini) ->
 --   >>> parseIniFile "[ONE]\nx = hello\n" $ sectionMb "TWO" (field "x")
 --   Right Nothing
 sectionMb :: Text -> SectionParser a -> IniParser (Maybe a)
-sectionMb name (SectionParser thunk) = IniParser $ ExceptT $ \(RawIni ini) ->
-  case lkp (normalize name) ini of
-    Nothing  -> return Nothing
-    Just sec -> Just `fmap` runExceptT thunk sec
+sectionMb name (SectionParser thunk) = IniParser $
+  ExceptT $ \(RawIni ini) ->
+    case lkp (normalize name) ini of
+      Nothing -> return Nothing
+      Just sec -> Just `fmap` runExceptT thunk sec
 
 -- | Find a named section in the INI file and parse it with the provided
 --   section parser, returning a default value if the section does not exist.
@@ -240,18 +251,19 @@ sectionMb name (SectionParser thunk) = IniParser $ ExceptT $ \(RawIni ini) ->
 --   >>> parseIniFile "[ONE]\nx = hello\n" $ sectionDef "TWO" "def" (field "x")
 --   Right "def"
 sectionDef :: Text -> a -> SectionParser a -> IniParser a
-sectionDef name def (SectionParser thunk) = IniParser $ ExceptT $ \(RawIni ini) ->
-  case lkp (normalize name) ini of
-    Nothing  -> return def
-    Just sec -> runExceptT thunk sec
+sectionDef name def (SectionParser thunk) = IniParser $
+  ExceptT $ \(RawIni ini) ->
+    case lkp (normalize name) ini of
+      Nothing -> return def
+      Just sec -> runExceptT thunk sec
 
 ---
 
 throw :: String -> StParser s a
-throw msg = ExceptT $ (\ _ -> Left msg)
+throw msg = ExceptT (\_ -> Left msg)
 
 getSectionName :: StParser IniSection Text
-getSectionName = ExceptT $ (\ m -> return (isName m))
+getSectionName = ExceptT (return . isName)
 
 rawFieldMb :: Text -> StParser IniSection (Maybe IniValue)
 rawFieldMb name = ExceptT $ \m ->
@@ -259,12 +271,16 @@ rawFieldMb name = ExceptT $ \m ->
 
 rawField :: Text -> StParser IniSection IniValue
 rawField name = do
-  sec   <- getSectionName
+  sec <- getSectionName
   valMb <- rawFieldMb name
   case valMb of
-    Nothing -> throw ("Missing field " ++ show name ++
-                      " in section " ++ show sec)
-    Just x  -> return x
+    Nothing ->
+      throw
+        ( "Missing field " ++ show name
+            ++ " in section "
+            ++ show sec
+        )
+    Just x -> return x
 
 getVal :: IniValue -> Text
 getVal = T.strip . vValue
@@ -294,7 +310,7 @@ fieldOf name parse = SectionParser $ do
   val <- rawField name
   case parse (getVal val) of
     Left err -> addLineInformation (vLineNo val) sec (throw err)
-    Right x  -> return x
+    Right x -> return x
 
 -- | Retrieve a field, returning a @Nothing@ value if it does not exist.
 --
@@ -320,10 +336,10 @@ fieldMbOf name parse = SectionParser $ do
   sec <- getSectionName
   mb <- rawFieldMb name
   case mb of
-    Nothing  -> return Nothing
+    Nothing -> return Nothing
     Just v -> case parse (getVal v) of
       Left err -> addLineInformation (vLineNo v) sec (throw err)
-      Right x  -> return (Just x)
+      Right x -> return (Just x)
 
 -- | Retrieve a field and supply a default value for if it doesn't exist.
 --
@@ -332,10 +348,11 @@ fieldMbOf name parse = SectionParser $ do
 --   >>> parseIniFile "[MAIN]\nx = hello\n" $ section "MAIN" (fieldDef "y" "def")
 --   Right "def"
 fieldDef :: Text -> Text -> SectionParser Text
-fieldDef name def = SectionParser $ ExceptT $ \m ->
-  case lkp (normalize name) (isVals m) of
-    Nothing -> return def
-    Just x  -> return (getVal x)
+fieldDef name def = SectionParser $
+  ExceptT $ \m ->
+    case lkp (normalize name) (isVals m) of
+      Nothing -> return def
+      Just x -> return (getVal x)
 
 -- | Retrieve a field, parsing it according to the given parser, and returning
 --   a default value if it does not exist. If the parser fails, then this will
@@ -352,10 +369,10 @@ fieldDefOf name parse def = SectionParser $ do
   sec <- getSectionName
   mb <- rawFieldMb name
   case mb of
-    Nothing  -> return def
+    Nothing -> return def
     Just v -> case parse (getVal v) of
       Left err -> addLineInformation (vLineNo v) sec (throw err)
-      Right x  -> return x
+      Right x -> return x
 
 -- | Retrieve a field and treat it as a boolean, failing if it
 --   does not exist.
@@ -377,7 +394,7 @@ fieldFlag name = fieldOf name flag
 --   >>> parseIniFile "[MAIN]\nx = yes\n" $ section "MAIN" (fieldFlagDef "y" False)
 --   Right False
 fieldFlagDef :: Text -> Bool -> SectionParser Bool
-fieldFlagDef name def = fieldDefOf name flag def
+fieldFlagDef name = fieldDefOf name flag
 
 ---
 
@@ -390,13 +407,18 @@ fieldFlagDef name def = fieldDefOf name flag def
 --   Left "Unable to parse \"hello\" as a value of type (Int,Int)"
 readable :: forall a. (Read a, Typeable a) => Text -> Either String a
 readable t = case readMaybe str of
-  Just v  -> Right v
-  Nothing -> Left ("Unable to parse " ++ show str ++
-                   " as a value of type " ++ show typ)
-  where str = T.unpack t
-        typ = typeRep prx
-        prx :: Proxy a
-        prx = Proxy
+  Just v -> Right v
+  Nothing ->
+    Left
+      ( "Unable to parse " ++ show str
+          ++ " as a value of type "
+          ++ show typ
+      )
+  where
+    str = T.unpack t
+    typ = typeRep prx
+    prx :: Proxy a
+    prx = Proxy
 
 -- | Try to use the "Read" instance for a numeric type to parse a value,
 --   failing with a human-readable error message if reading fails.
@@ -434,15 +456,15 @@ string = return . fromString . T.unpack
 --   Left "Unable to parse \"That's a secret!\" as a boolean"
 flag :: Text -> Either String Bool
 flag s = case T.toLower s of
-  "true"  -> Right True
-  "yes"   -> Right True
-  "t"     -> Right True
-  "y"     -> Right True
+  "true" -> Right True
+  "yes" -> Right True
+  "t" -> Right True
+  "y" -> Right True
   "false" -> Right False
-  "no"    -> Right False
-  "f"     -> Right False
-  "n"     -> Right False
-  _       -> Left ("Unable to parse " ++ show s ++ " as a boolean")
+  "no" -> Right False
+  "f" -> Right False
+  "n" -> Right False
+  _ -> Left ("Unable to parse " ++ show s ++ " as a boolean")
 
 -- | Convert a reader for a value into a reader for a list of those
 --   values, separated by a chosen separator. This will split apart
@@ -462,10 +484,12 @@ flag s = case T.toLower s of
 --   Right ["/bin","/usr/bin"]
 --   >>> listWithSeparator "," number "7 8 9" :: Either String [Int]
 --   Left "Unable to parse \"7 8 9\" as a value of type Int"
-listWithSeparator :: (IsList l)
-                  => Text
-                  -> (Text -> Either String (Item l))
-                  -> Text -> Either String l
+listWithSeparator ::
+  (IsList l) =>
+  Text ->
+  (Text -> Either String (Item l)) ->
+  Text ->
+  Either String l
 listWithSeparator sep rd =
   fmap fromList . mapM (rd . T.strip) . T.splitOn sep
 
